
__all__ = ['unet_v',
           'unet_v2',
           'hourglass_wres',
           'hourglass_wores',
           'unet_v_synth',
           'unet_v2_synth',
           'hourglass_wres_synth',
           'hourglass_wores_synth',
           'unet_v_tr',
           'hourglass_wres_tr',
           'unet_v_k5',
           'hourglass_wres_k5',
           'unet_v_patch',
           'hourglass_wres_patch']